﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using Microsoft.SqlServer.Management.Smo;
using EA;
using EAUtil;

namespace EaWorkflowDesignerAddin.Ui
{
    public partial class MainForm : Form
    {
        private Repository _repository;
        private Diagram _stateMachineDiagram = null;


        #region Properties

        public MainformPresenter Presenter { get; set; }

        public Repository Repository
        {
            get { return _repository; }
            set
            {
                _repository = value;
                PopulateStateMachineDiagramList();
            }
        }

        #endregion

        public MainForm()
        {
            InitializeComponent();
        }

        public void ShowError(string msg)
        {
            MessageBox.Show(msg);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void PopulateDatabaseList()
        {
            cmbWorkflowDatabase.DataSource = null;
            string serverName = txtBoxDatabaseServer.Text;
            if (!String.IsNullOrEmpty(serverName))
            {
                try
                {
                    Cursor = Cursors.WaitCursor;
                    Server server = new Server(serverName);
                    DatabaseCollection dbCollection = server.Databases;
                    if (dbCollection != null)
                    {
                        List<string> dbNames = new List<string>();
                        foreach (Database db in dbCollection)
                        {
                            dbNames.Add(db.Name);
                        }
                        cmbWorkflowDatabase.DataSource = dbNames;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                }
                finally
                {
                    Cursor = Cursors.Default;
                }
            }
        }

        private void txtBoxDatabaseServer_Leave(object sender, EventArgs e)
        {
            PopulateDatabaseList();
        }

        private void txtBoxDatabaseServer_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                PopulateDatabaseList();
            }
        }

        private void PopulateStateMachineDiagramList()
        {
            cmbStateMachineDiagram.DataSource = null;
            List<Diagram> stateMachineDiagrams = new List<Diagram>();
            foreach (Package package in Repository.Models)
            {
                EaUtil.GetAllDiagramsInPackage(stateMachineDiagrams, package);
            }
            // Convert to list of DiagramListEntry:
            List<DiagramListEntry> statechartDiagramList = new List<DiagramListEntry>();
            foreach (Diagram diagram in stateMachineDiagrams)
            {
                if (diagram.Type == "Statechart")
                {
                    statechartDiagramList.Add(new DiagramListEntry(diagram.Name, diagram));
                }
            }
            cmbStateMachineDiagram.DataSource = statechartDiagramList;
            cmbStateMachineDiagram.DisplayMember = "Name";
        }

        private void cmbStateMachineDiagram_SelectedValueChanged(object sender, EventArgs e)
        {
            _stateMachineDiagram = null;
            btnGenerate.Enabled = false;
            if (cmbStateMachineDiagram.SelectedItem == null) return;

            Diagram diagram = ((DiagramListEntry)cmbStateMachineDiagram.SelectedItem).Diagram;
            if (diagram.Type == "Statechart")
            {
                btnGenerate.Enabled = true;
                _stateMachineDiagram = diagram;
            }
        }

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            if (_stateMachineDiagram == null) return;
            Presenter.GenerateWorkflowFromDiagram(
                txtBoxDatabaseServer.Text,
                cmbWorkflowDatabase.Text,
                _stateMachineDiagram,
                _repository);
        }


    }

    class DiagramListEntry
    {
        public string Name { get; set; }
        public Diagram Diagram { get; set; }
        public DiagramListEntry(string name, Diagram diagram)
        {
            Name = name;
            Diagram = diagram;
        }

    }
}
